# -*- coding: utf-8 -*-
"""
Created on Wed Feb 17 14:31:04 2021

@author: Andreas
"""

import numpy as np

from .jacobian import jacobian
from .newtonxt import newtonxt
from . import printinfo


def solve(
    fun,
    x0,
    lpf0,
    jac=None,
    args=(None,),
    dxmax=0.05,
    dlpfmax=0.05,
    control0="lpf",
    jacmode=2,
    jaceps=1e-6,
    maxsteps=80,
    maxcycles=4,
    maxiter=20,
    tol=1e-6,
):
    "Numeric continuation algorithm."

    # init initial control component
    if control0 == "lpf":
        j0 = 1 + len(x0)
    else:
        j0 = control0

    # init y=(x,l)-combined quantities
    y0 = np.append(x0, lpf0)
    dymax = np.append(np.ones_like(x0) * dxmax, dlpfmax)

    # init list for results
    Res = []

    # init list for results
    Res = [newtonxt(fun, jac, y0, j0, dymax, jacmode, jaceps, args, maxiter=0, tol=tol)]

    # Step loop.
    for step in 1 + np.arange(maxsteps):
        ## pre-identification of control component
        res = newtonxt(
            fun, jac, y0, j0, dymax, jacmode, jaceps, args, maxiter=1, tol=tol
        )
        printinfo.stepheader(step)
        # Increment loop.
        for cycl in 1 + np.arange(maxcycles):
            res = newtonxt(
                fun, jac, y0, j0, dymax, jacmode, jaceps, args, maxiter=maxiter, tol=tol
            )
            printinfo.cycle(cycl, res.message, j0, res.control)

            if res.success:
                if res.control == j0:
                    y0 = res.x
                    Res.append(res)
                    printinfo.stepfinal(res.x, res.control, res.fun)
                    break
                else:
                    if cycl == maxcycles:
                        print("\nERROR. Control component changed in last cycle.")
                        print("       Possible solution: Reduce stepwidth.")
                        res.success = False
                    else:
                        # next cycle
                        j0 = res.control
            else:
                break

        if not res.success:
            print("\nERROR. Numerical continuation stopped.")
            break
    return Res
